/*  OpenEngine: A free open source rendering system for DirectX.
	Copyright (C) 2002 Simon Brown

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

namespace oe {

template<typename I>
inline void ComPtr<I>::Reset(I* const& p_) throw()
{
	if(p_)
		p_->AddRef();
	Free();
	p = p_;
}

template<typename I>
inline void ComPtr<I>::Free() throw()
{
	if(p)
	{
		p->Release();
		p = 0;
	}
}

template<typename I>
ComPtr<I>::ComPtr() throw() : p(0) {}

template<typename I>
ComPtr<I>::~ComPtr() throw()
{
	Free();
}

template<typename I>
ComPtr<I>::ComPtr(I* const& p_) throw() : p(p_) {}

template<typename I>
ComPtr<I>::ComPtr(const ComPtr<I>& ptr) throw() : p(0) 
{
	Reset(ptr.p);
}

template<typename I>
ComPtr<I>& ComPtr<I>::operator=(const ComPtr<I>& ptr) throw()
{
	Reset(ptr.p);
	return *this;
}

template<typename I>
I* const& ComPtr<I>::Get() const throw() { return p; }

template<typename I>
I* const& ComPtr<I>::operator->() const throw(Exception)
{
	if(!p)
		throw Exception("ComPtr::operator->(): Access violation.");
	return p;
}

template<typename I>
ComPtr<I>::operator bool() const throw() { return p != 0; }

template<typename I>
I** ComPtr<I>::AddressOf() throw()
{
	if(p)
		std::cerr << "ComPtr::AddressOf(): Non-zero pointer addressed - possible memory leak." << std::endl;
	return &p;
}

template<typename I>
inline bool operator<(const ComPtr<I>& l, const ComPtr<I>& r) throw()
{
    return l.Get() < r.Get();
}

} // namespace oe