/*  OpenEngine: A free open source rendering system for DirectX.
	Copyright (C) 2002 Simon Brown

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"

namespace oe {

Exception::Exception(const std::string& excuse_) throw() : excuse(excuse_) 
{
	std::cerr << "Exception: " << excuse << std::endl;
}

const std::string& Exception::GetExcuse() const throw() 
{ 
	return excuse; 
}

std::string FormatDirectXError(const HRESULT& hr) throw()
{
	std::ostringstream ss;
	ss << DXGetErrorString8(hr) << " (" << hr << ")";
	return DXGetErrorString8(hr);
}

std::string FormatWin32Error(const ulong& id) throw()
{
	char* msg = 0;
	FormatMessage( 
		FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
		0,
		id,
		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
		reinterpret_cast<char*>(&msg),
		0,
		0 
	);
	std::ostringstream ss;
	ss << msg << " (" << id << ")";
	LocalFree(msg);
    return ss.str();
}

Exception CreateDirectXException(const char* const& cause, const HRESULT& hr) throw()
{
	std::ostringstream ss;
	ss << cause << FormatDirectXError(hr) << ".";
	return Exception(ss.str());
}

Exception CreateWin32Exception(const char* const& cause, const ulong& id) throw()
{
	std::ostringstream ss;
	ss << cause << FormatWin32Error(id) << ".";
	return Exception(ss.str());
}

} // namespace oe
