/*  OpenEngine: A free open source rendering system for DirectX.
	Copyright (C) 2002 Simon Brown

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#pragma once

namespace oe {

//! A simple exception class for error reporting.
class Exception
{
private:
	std::string excuse;

public:
    //! Creates a new exception with the given message.
	Exception(const std::string& excuse_) throw();

    //! Gets the message for this exception.
	const std::string& GetExcuse() const throw();
};

//! Looks up the description for a DirectX HRESULT error code.
std::string FormatDirectXError(const HRESULT& hr) throw();

//! Looks up the description for a win32 error code.
std::string FormatWin32Error(const ulong& id) throw();

//! Creates an exception from a DirectX HRESULT error code.
/*! Creates an exception from a DirectX HRESULT error code. The string in \a cause
    is used as a prefix for the exception message.
*/
Exception CreateDirectXException(const char* const& cause, const HRESULT& hr) throw();

//! Creates an exception for a win32 error code.
/*! Creates an exception for a win32 error code. The string in \a cause
    is used as a prefix for the exception message.
*/
Exception CreateWin32Exception(const char* const& cause, const ulong& id) throw();

} // namespace oe
