/*  OpenEngine: A free open source rendering system for DirectX.
	Copyright (C) 2002 Simon Brown

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#pragma once

namespace oe {

//! Shared font resource wrapper.
class Font
{
private:
	Texture texture;
	boost::shared_array<uchar> widths;
	uchar height;

protected:
	//! Creates a new font given a texture and character widths.
	Font(const Texture& texture_, const boost::shared_array<uchar>& widths_, const uchar& height_) throw();

public:
	//! Create an empty font.
	Font() throw();

	const Texture& GetTexture() const throw();						//!< Gets the font textures.
	const uchar& GetWidth(const uchar& c) const throw(Exception);	//!< Gets the character width for a particular character.
	const uchar& GetHeight() const throw();							//!< Gets the font height in the texture.

	//! Allows the Graphics class to create font objects.
	friend class Graphics;
};

} // namespace oe
