/*  OpenEngine: A free open source rendering system for DirectX.
	Copyright (C) 2002 Simon Brown

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"

namespace oe {

FrameTimer::FrameTimer() throw(Exception) : frameTime(_R(0))
{
	memset(&marker, 0, sizeof(LARGE_INTEGER));
	// get the performance counter resolution for this system
	LARGE_INTEGER frequency;
	memset(&frequency, 0, sizeof(LARGE_INTEGER));
	if(!QueryPerformanceFrequency(&frequency) || frequency.QuadPart < (LONGLONG)10000)
		throw Exception("FrameTimer::FrameTimer(): Insufficient performance query resolution on this system.");
	performanceUnit = _R(1) / _R(frequency.QuadPart);
}

real_t FrameTimer::Mark() throw()
{
	// if this frame should be ignored, or this is the first call to mark, just update the marker
	if(ignoreThisFrame || marker.QuadPart == (LONGLONG)0)
	{
		QueryPerformanceCounter(&marker);
		ignoreThisFrame = false;
		return frameTime;
	}
	
	// update the smoothed frame time
	LARGE_INTEGER temp;
	QueryPerformanceCounter(&temp);
	real_t elapsedTime = _R(temp.QuadPart - marker.QuadPart) * performanceUnit;
	memcpy(&marker, &temp, sizeof(LARGE_INTEGER));
	
	// if this is the first timed frame then store it, otherwise smoothly update
	if(frameTime == _R(0))
		frameTime = elapsedTime;
	else
		frameTime = elapsedTime * _R(0.05) + frameTime * _R(0.95);
	
	// return the smoothed frame time
	return frameTime;
}

void FrameTimer::IgnoreThisFrame() throw()
{
	ignoreThisFrame = true;
}

} // namespace oe
