/*  OpenEngine: A free open source rendering system for DirectX.
	Copyright (C) 2002 Simon Brown

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#pragma once

namespace oe {

//! Provides a smooth message-aware frame timer.
class FrameTimer : boost::noncopyable
{
private:
	LARGE_INTEGER marker;
	real_t performanceUnit, frameTime;
	bool ignoreThisFrame;
	
protected:
	//! Creates a new frame timer if the system performance counter is accurate enough.
	FrameTimer() throw(Exception);
	
public:
	//! Marks the end of this frame and the start of the next.
	real_t Mark() throw();
	
	//! Tells the timer updater to ignore this frame time.
	void IgnoreThisFrame() throw();
	
	friend class Graphics; //!< Allows the graphics class to create a frame timer.
};

} // namespace oe
