/*  OpenEngine: A free open source rendering system for DirectX.
	Copyright (C) 2002 Simon Brown

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

namespace oe {

template<typename V>
VertexBuffer<V> Graphics::CreateVertexBuffer(const ulong& numVertices, const OE_BUFFERTRAITS& traits) const throw(Exception)
{
	// log creation
	std::cout << "Creating vertex buffer (" << numVertices * sizeof(V) << " bytes)..." << std::endl;
	// get the traits 
	ulong usageTraits, lockTraits;
	GetBufferTraits(traits, usageTraits, lockTraits);
	// create the buffer
	VertexBuffer<V> buffer(lockTraits);
	HRESULT hr = device->CreateVertexBuffer(
		numVertices * sizeof(V), 
		usageTraits, 
		0, 
		D3DPOOL_DEFAULT, 
		buffer.AddressOf()
	);
	if(FAILED(hr))
		throw CreateDirectXException("Graphics::CreateVertexBuffer(): CreateVertexBuffer failed: ", hr);
	// return a copy
	return buffer;
}

template<typename V>
void Graphics::SetStreamSource(const ulong& stream, const VertexBuffer<V>& buffer) const throw(Exception)
{
	if(!buffer.Get())
		throw Exception("Graphics::SetStreamSource(): Null stream.");
	HRESULT hr = device->SetStreamSource(stream, buffer.Get(), sizeof(V));
	if(FAILED(hr))
		throw CreateDirectXException("Graphics::SetStreamSource(): SetStreamSource failed: ", hr);
}

template<typename I>
void Graphics::SetTexture(const ulong& stage, const ComPtr<I>& texture) const throw(Exception)
{
	if(!texture.Get())
		throw Exception("Graphics::SetTexture(): Null texture.");
	HRESULT hr = device->SetTexture(stage, texture.Get());
	if(FAILED(hr))
		throw CreateDirectXException("Graphics::SetTexture(): SetTexture failed: ", hr);
}

} // namespace oe