/*  OpenEngine: A free open source rendering system for DirectX.
	Copyright (C) 2002 Simon Brown

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#pragma once

namespace oe {

//! Displays a graphics loader dialog to choose an adapter and display mode.
class GraphicsLoader
{
private:
	HINSTANCE instance;
	ComPtr<IDirect3D8> direct3d;
	D3DFORMAT depthStencilFormat;

	static GraphicsLoader* Get(const HWND& dialog) throw(Exception);
	static INT_PTR CALLBACK DialogProc(HWND dialog, UINT msg, WPARAM wParam, LPARAM lParam) throw();

	long adapter;
	bool fullscreen;
	long mode;

	//! Used interally to index a display mode.
    class DisplayMode { public: D3DDISPLAYMODE mode; ulong index; };
    //! Used internally as an stl functor to sort indexed display modes by pixel area.
	class mode_comp { public: bool operator()(DisplayMode* const& l, DisplayMode* const& r) const throw(); };
	
	void EnumerateAdapters(const HWND& dialog) throw(Exception);
	void EnumerateModes(const HWND& dialog) throw(Exception);
	void Enable(const HWND& dialog, const bool& enable) throw();

public:
    //! Creates a loader dialog.
    /*! Creates a loader dialog.
        \param instance_    The application instance.
        \param direct3d_    A valid COM pointer to a IDirect3D8 object.
        \param depthStencilFormat_  The required depth stencil buffer format, which can be D3DFMT_UNKNOWN for no buffer.
    */
	GraphicsLoader(const HINSTANCE& instance_, const ComPtr<IDirect3D8>& direct3d_, const D3DFORMAT& depthStencilFormat_) throw();

    //! Shows the loader.
    bool Show(long& adapter_, D3DPRESENT_PARAMETERS& pp) throw(Exception);
};

} // namespace oe
