/*  OpenEngine: A free open source rendering system for DirectX.
	Copyright (C) 2002 Simon Brown

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#pragma once

namespace oe {

//! Template index buffer wrapper.
template<typename I>
class IndexBuffer : public ComPtr<IDirect3DIndexBuffer8>
{
private:
	ulong lockTraits;

public:
	//! The type of the index in this buffer.
	typedef I Index;

	//! The type of SafeLock returned by this index buffer.
	typedef SafeLock<I, IDirect3DIndexBuffer8> SafeLock;

	//! Creates an empty index buffer.
	IndexBuffer() throw() : lockTraits(0) {}

	//! Creates an empty index buffer with the given lock traits.
	explicit IndexBuffer(const ulong& lockTraits_) throw() : lockTraits(lockTraits_) {}

    //! Locks the entire buffer.
	SafeLock Lock() throw(Exception)
	{ 
		return Lock(0, 0); 
	}

    //! Locks a subset of the buffer.
    SafeLock Lock(const ulong& startOffset, const ulong& numIndicesToLock) throw(Exception)
	{
		if(!GetNoAddRef())
			throw Exception("IndexBuffer::Lock(): Access violation.");
		// lock (with discard) the given region of the vertex buffer
		I* indices = 0;
		HRESULT hr = GetNoAddRef()->Lock(
			startOffset * sizeof(I), 
			numIndicesToLock * sizeof(I), 
			reinterpret_cast<BYTE**>(&indices), 
			lockTraits
		);
		if(FAILED(hr))
			throw CreateDirectXException("IndexBuffer::Lock(): Lock failed: ", hr);
		// return it
		return SafeLock(indices, *this);
	}
};

} // namespace oe
