/*  OpenEngine: A free open source rendering system for DirectX.
	Copyright (C) 2002 Simon Brown

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"

namespace oe {

Input::Input(const Graphics& graphics) throw(Exception)
{
	// check that the graphics object has been created properly
	if(!graphics.GetWindow())
		throw Exception("Input::Input(): Graphics device has no window handle.");

	// construct the base DirectInput8 device
	HRESULT hr = DirectInput8Create(
		graphics.GetInstance(), 
		DIRECTINPUT_VERSION, 
		IID_IDirectInput8, 
		reinterpret_cast<void**>(dinput.AddressOf()), 
		0
	);
	if(!dinput)
		throw CreateDirectXException("Input::Input(): DirectInput8Create failed: ", hr);

	// create a keyboard device
	hr = dinput->CreateDevice(GUID_SysKeyboard, keyboard.AddressOf(), 0);
	if(FAILED(hr))
		throw CreateDirectXException("Input::Input(): CreateDevice failed: ", hr);

	// set the keyboard cooperative level
	hr = keyboard->SetCooperativeLevel(graphics.GetWindow(), DISCL_NONEXCLUSIVE | DISCL_FOREGROUND);
	if(FAILED(hr))
		throw CreateDirectXException("Input::Input(): SetCooperativeLevel failed: ", hr);

	//set the keyboard data format
	hr = keyboard->SetDataFormat(&c_dfDIKeyboard);
	if(FAILED(hr))
		throw CreateDirectXException("Input::Input(): SetDataFormat failed: ", hr);
	
	// attempt to aquire the keyboard
	keyboard->Acquire();
	memset(keyboardData, 0, 256);
}

void Input::GetImmediateKeyboardData() throw()
{
	// get the immediate keyboard data
	HRESULT hr = keyboard->GetDeviceState(256, keyboardData);
	if(FAILED(hr))
		memset(keyboardData, 0, 256);
	// attempt to reacquire the device if it has been lost
	if(hr == DIERR_INPUTLOST || hr == DIERR_NOTACQUIRED)
		keyboard->Acquire();
}

bool Input::IsKeyDown(uchar key) const throw()
{
	return ((keyboardData[key] & 0x80) != 0);
}

} // namespace oe