/*  OpenEngine: A free open source rendering system for DirectX.
	Copyright (C) 2002 Simon Brown

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#pragma once 

namespace oe {

//! A wrapper around DirectX input devices.
class Input : boost::noncopyable
{
private:
	ComPtr<IDirectInput8> dinput;
	ComPtr<IDirectInputDevice8> keyboard;
	uchar keyboardData[256];

public:
	//! Creates a new device with access to the keyboard.
	Input(const Graphics& graphics) throw(Exception);

	//! Updates the keyboard data from the current key states.
	void GetImmediateKeyboardData() throw();
	
	//! Tests whether a given key is currently pressed.
	bool IsKeyDown(uchar key) const throw();
};

} // namespace oe