/*  OpenEngine: A free open source rendering system for DirectX.
	Copyright (C) 2002 Simon Brown

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"

namespace oe {

Projection::Projection() throw() : fovRatio(_R(2)), aspectRatio(_R(1)), zNear(_R(1)), zFar(_R(1000)) 
{
	CreateTransform();
}

void Projection::CreateTransform() throw()
{
	// scale
	transform.e11 = fovRatio;
	transform.e22 = fovRatio * aspectRatio;
	// w-friendly perspective
	transform.e33 = zFar / (zFar - zNear);
	transform.e34 = -transform.e33 * zNear;
	transform.e43 = _R(1);
	// zero the rest
	transform.e12 = transform.e13 = transform.e14 = 
		transform.e21 = transform.e23 = transform.e24 = 
		transform.e31 = transform.e32 = 
		transform.e41 = transform.e42 = transform.e44 = _R(0);
}

const Matrix4x4& Projection::GetTransform() const throw()
{
	return transform;
}

} // namespace oe