/*  OpenEngine: A free open source rendering system for DirectX.
	Copyright (C) 2002 Simon Brown

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#pragma once

namespace oe {

//! A projection of a 3D scene into 2D space.
class Projection
{
private:
	Matrix4x4 transform;

public:
	real_t fovRatio; 	//!< The field of view ratio, \f$ \frac{2 \times distance}{width} \f$ .
	real_t aspectRatio;	//!< The aspect ratio, \f$ \frac{width}{height} \f$ .
	real_t zNear;		//!< The near \c z plane.
	real_t zFar;		//!< The far \c z plane.

	//! Creates the default projection.
	Projection() throw();

	//! Creates the projection transform using the current properties.
	void CreateTransform() throw();
	//! Get the current projection transform as a 4x4 matrix.
	const Matrix4x4& GetTransform() const throw();
};

} // namespace oe