/*  OpenEngine: A free open source rendering system for DirectX.
	Copyright (C) 2002 Simon Brown

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"

namespace oe {

bool RenderWindow::active = false;
bool RenderWindow::sizemoved = false;

bool RenderWindow::IsActive() throw() 
{ 
	return active; 
}

bool RenderWindow::WasSizeMoved() throw() 
{ 
	bool temp = sizemoved; 
	sizemoved = false; 
	return temp; 
}

LRESULT CALLBACK RenderWindow::RenderWindowProc(HWND window, UINT msg, WPARAM wParam, LPARAM lParam) throw()
{
	switch(msg)
	{
	case WM_KEYDOWN:
		if(wParam == VK_ESCAPE)
			DestroyWindow(window);
		break;

	case WM_EXITSIZEMOVE:
		sizemoved = true;
		break;
	
	case WM_ACTIVATE:
		active = (wParam != WA_INACTIVE);
		break;

	case WM_DESTROY:
		PostQuitMessage(0);
		break;

	case WM_PAINT:
		ValidateRect(window, 0);
		return 0;
	}
	return DefWindowProc(window, msg, wParam, lParam);
}

RenderWindow::RenderWindow() throw () : handle(0) {}

RenderWindow::~RenderWindow() throw()
{
	if(IsWindow(handle))
	{
		DestroyWindow(handle);
		handle = 0;
	}
}

void RenderWindow::Create(const HINSTANCE& instance, 
						  const char* const& title, 
						  const long& width, 
						  const long& height) throw(Exception)
{
	// sanity check
	if(handle)
		throw Exception("RenderWindow::Create(): This instance already has a render window.");
		
	// register the window class
	static const char* const renderWindowClass = "OpenEngine Render Target Window Class";
	WNDCLASSEX wcx = {
		sizeof(WNDCLASSEX),
		CS_OWNDC, 
		RenderWindowProc,
		0, 
		0, 
		instance, 
		0, 
		LoadCursor(0, IDC_ARROW), 
		(HBRUSH)(COLOR_WINDOW + 1), 
		0, 
		renderWindowClass,
		0
	};
	if(!RegisterClassEx(&wcx))
		throw CreateWin32Exception("RenderWindow::Create(): RegisterClassEx failed: ", GetLastError());

	// adjust the window size so that the client area matches the arguments
	RECT size = { 0, 0, width, height };
	AdjustWindowRect(&size, WS_SYSMENU | WS_CAPTION | WS_BORDER, FALSE);
	
	// create the window
	handle = CreateWindow(
		renderWindowClass, 
		title, 
		WS_SYSMENU | WS_CAPTION | WS_BORDER, 
		CW_USEDEFAULT, 
		0, 
		size.right - size.left, 
		size.bottom - size.top, 
		0, 
		0, 
		instance, 
		0
	);
	if(!handle)
		throw CreateWin32Exception("RenderWindow::Create(): CreateWindow failed: ", GetLastError());

	// show the window immediately
	ShowWindow(handle, SW_SHOWNORMAL);
	UpdateWindow(handle);
}

const HWND& RenderWindow::Get() const throw() 
{ 
	return handle; 
}

} // namespace oe
