/*  OpenEngine: A free open source rendering system for DirectX.
	Copyright (C) 2002 Simon Brown

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#pragma once

namespace oe {

//! A simple render target window wrapper.
class RenderWindow : boost::noncopyable
{
private:
	HWND handle;

	static bool active;
	static bool sizemoved;
	static LRESULT CALLBACK RenderWindowProc(HWND window, UINT msg, WPARAM wParam, LPARAM lParam) throw();

protected:
	static bool IsActive() throw();        //!< Returns \c true if the window is active.
	static bool WasSizeMoved() throw();    //!< Returns \c true if the window has been moved or sized since the last call to WasSizeMoved().

	//! Initialises the class.
    RenderWindow() throw();
    
	//! Creates a new window inside the class.
    void Create(const HINSTANCE& instance, const char* const& title, const long& width, const long& height) throw(Exception);
	
public:
    //! Destroys the window.
	~RenderWindow() throw();

    //! Returns the window handle.
	const HWND& Get() const throw();

	friend class Graphics; //!< Allows the Graphics class to create a render window
};

} // namespace oe
