/*  OpenEngine: A free open source rendering system for DirectX.
	Copyright (C) 2002 Simon Brown

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

namespace oe {

template<typename T, typename I>
SafeLockDeleter<T, I>::SafeLockDeleter(const ComPtr<I>& buffer_) throw() : buffer(buffer_) {}

template<typename T, typename I>
void SafeLockDeleter<T, I>::operator()(T* const&) throw()
{
	buffer->Unlock();
}

template<typename T, typename I>
SafeLock<T, I>::SafeLock(T* const& lock, const ComPtr<I>& buffer) throw()
	: SharedHandle<T*, SafeLockDeleter<T, I> >(lock, SafeLockDeleter<T, I>(buffer)) {}

template<typename T, typename I>
void SafeLock<T, I>::Unlock() throw()
{
	Free();
}

template<typename T, typename I>
T& SafeLock<T, I>::operator[](const ulong& index) throw(Exception)
{
	if(!Get())
		throw Exception("SafeLock::operator[]: Access violation.");
	return Get()[index];
}

} // namespace oe