/*  OpenEngine: A free open source rendering system for DirectX.
	Copyright (C) 2002 Simon Brown

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

namespace oe {

template<typename H, typename Deleter>
inline void SharedHandle<H, Deleter>::Reset(const SharedHandle<H, Deleter>& sh) throw()
{
	if(sh.count)
		++*sh.count;
	Free();
	handle = sh.handle;
	count = sh.count;
	deleter = sh.deleter;
}

template<typename H, typename Deleter>
inline void SharedHandle<H, Deleter>::Free() throw()
{
	if(count)
	{
		if(--*count == 0)
		{
			deleter(handle);
			delete count;
		}
		handle = 0;
		count = 0;
	}
}

template<typename H, typename Deleter>
SharedHandle<H, Deleter>::SharedHandle() throw() : handle(0), count(0) {}

template<typename H, typename Deleter>
SharedHandle<H, Deleter>::~SharedHandle() throw()
{
	Free();
}

template<typename H, typename Deleter>
SharedHandle<H, Deleter>::SharedHandle(const H& handle_, const Deleter& deleter_) throw() 
  : handle(handle_), count(0), deleter(deleter_)
{
	if(handle)
		count = new ulong(1);
}

template<typename H, typename Deleter>
SharedHandle<H, Deleter>::SharedHandle(const SharedHandle<H, Deleter>& sh) throw() : handle(0), count(0)
{
	Reset(sh);
}

template<typename H, typename Deleter>
SharedHandle<H, Deleter>& SharedHandle<H, Deleter>::operator=(const SharedHandle<H, Deleter>& sh) throw()
{
	Reset(sh);
	return *this;
}

template<typename H, typename Deleter>
inline const H& SharedHandle<H, Deleter>::Get() const throw() 
{ 
	return handle; 
}

template<typename U, typename V>
std::ostream& operator<<(std::ostream& os, const SharedHandle<U, V>& sh) throw()
{
	if(sh.count)
		return cout << sh.handle << " (" << *sh.count << ")";
	else
		return cout << sh.handle << " (-)";
}

} // namespace oe