/*  OpenEngine: A free open source rendering system for DirectX.
	Copyright (C) 2002 Simon Brown

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#pragma once

namespace oe {

//! Buffers text as triangle primitives.
class TextBuffer : boost::noncopyable
{
private:
	VertexBuffer<GuiVertex> vertices;
	ulong capacity;
	ulong used;
	
	Font font;
	short x, y;

public:
	//! Creates a new text buffer of given capacity.
	TextBuffer(const Graphics& graphics, const ulong& capacity_) throw(Exception);

	//! Clears the buffer.
	void Clear() throw();
	
	//! Sets the current text position.
	void SetPosition(const short& x_, const short& y_) throw();
	
	//! Sets the current font.
	void SetFont(const Font& font_) throw();

	//! Prints a line of text into the buffer.
	void Println(const std::string& text) throw(Exception);

	//! Prints an empty line.
	void Println() throw();
	
	//! Draws all buffered text.
	void DrawPrimitive(const Graphics& graphics) const throw(Exception);
};

} // namespace oe
