/*  OpenEngine: A free open source rendering system for DirectX.
	Copyright (C) 2002 Simon Brown

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#pragma once

namespace oe {

//! Template vertex buffer wrapper.
template<typename V>
class VertexBuffer : public ComPtr<IDirect3DVertexBuffer8>
{
private:
	ulong lockTraits;

public:
	//! The type of the vertex in this buffer.
	typedef V Vertex;

	//! The type of SafeLock returned by this vertex buffer.
	typedef SafeLock<V, IDirect3DVertexBuffer8> SafeLock;

	//! Creates an empty vertex buffer resource.
	VertexBuffer() throw() : lockTraits(0) {}

	//! Create a new vertex buffer with the given locking traits.
	explicit VertexBuffer(const ulong& lockTraits_) throw() : lockTraits(lockTraits_) {}

    //! Locks the entire buffer.
	SafeLock Lock() throw(Exception)
	{	
		return Lock(0, 0); 
	}

    //! Locks a subset of the buffer.
	SafeLock Lock(const ulong& startOffset, const ulong& numVerticesToLock) throw(Exception)
	{
		if(!Get())
			throw Exception("VertexBuffer::Lock(): Access violation");
		// lock (with discard) the given region of the vertex buffer
		V* vertices = 0;
		HRESULT hr = Get()->Lock(
			startOffset * sizeof(V), 
			numVerticesToLock * sizeof(V), 
			reinterpret_cast<BYTE**>(&vertices), 
			lockTraits
		);
		if(FAILED(hr))
			throw CreateDirectXException("VertexBuffer::Lock(): Lock failed: ", hr);
		// return it
		return SafeLock(vertices, *this);
	}
};

} // namespace oe
