/*  OpenEngine: A free open source rendering system for DirectX.
	Copyright (C) 2002 Simon Brown

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#pragma once 

namespace oe {

//! Deletes a vertex shader once no longer referenced.
class VertexShaderDeleter
{
private:
	ComPtr<IDirect3DDevice8> device;

public:
	//! Creates an empty vertex shader deleter.
	VertexShaderDeleter() throw();

	//! Creates a new deleter for a particular device.
	explicit VertexShaderDeleter(const ComPtr<IDirect3DDevice8>& device_) throw();

	//! Deletes the handle using the encapsulated device.
	void operator()(const ulong& handle) throw();
};

//! Holds a vertex shader as a shared resource.
class VertexShader : public SharedHandle<ulong, VertexShaderDeleter>
{
public:
	//! Creates an empty vertex shader resource.
	VertexShader() throw();

	//! Creates a new vertex shader object around the given handle.
	VertexShader(const ulong& handle_, const ComPtr<IDirect3DDevice8>& device_) throw();
};

} // namespace oe
