/*  OpenEngine: A free open source rendering system for DirectX.
	Copyright (C) 2002 Simon Brown

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"

namespace oe {

#ifndef DOXYGEN_PARSING
namespace gui {
#include "textureVS.inl"
#include "guiPS.inl"
}
#endif // ndef DOXYGEN_PARSING
	
ulong GuiVertex::decl[] = { 
	D3DVSD_STREAM(0),
	D3DVSD_REG(0, D3DVSDT_SHORT2), // 2d position
	D3DVSD_REG(3, D3DVSDT_FLOAT2), // texture co-ordinate
	D3DVSD_END()
};

VertexShader CreateGuiVertexShader(const Graphics& graphics) throw(Exception)
{
	return graphics.CreateVertexShader(GuiVertex::decl, gui::dwTextureVertexShader);
}

PixelShader CreateGuiPixelShader(const Graphics& graphics) throw(Exception)
{
	return graphics.CreatePixelShader(gui::dwGuiPixelShader);
}

void CreateGuiTransform(const Graphics& graphics, Matrix4x4& transform) throw()
{
	transform.e11 = _R(2) / _R(graphics.GetTargetWidth() - 1);
	transform.e14 = _R(-1);
	transform.e22 = -_R(2) / _R(graphics.GetTargetHeight() - 1);
	transform.e24 = _R(1);
	transform.e34 = _R(0.5);
	transform.e12 = transform.e13
		= transform.e21 = transform.e23
		= transform.e31 = transform.e32 = transform.e33
		= transform.e41 = transform.e42 = transform.e43 = _R(0); 
	transform.e44 = _R(1); 
}

} // namespace oe
