/*  OpenEngine: A free open source rendering system for DirectX.
	Copyright (C) 2002 Simon Brown

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#pragma once

// C++ exception specification ignored except to indicate a function is not __declspec(nothrow)
#pragma warning(disable: 4290) 

//! The OpenEngine namespace.
/*! The OpenEngine namespace. All classes used in the engine reside here.
*/
namespace oe {

// forward declarations
class Graphics;
class Exception;

//! Shows a modal message box containing the exception message.
extern inline void ReportException(const Exception& e) throw();

} // namespace oe

//! The OpenEngine main function.
/*! The OpenEngine main function. This function is called by the system to run
    the application. If this function must be defined.
    \param graphics An initialised graphics object.
    \throw          Exception can be thrown to indicate an application error. The user
    will be shown a message box with the error before the system exits.
*/
void oe_main(oe::Graphics& graphics) throw(oe::Exception);
