/*  OpenEngine: A free open source rendering system for DirectX.
	Copyright (C) 2002 Simon Brown

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#pragma once

namespace oe {

#ifndef DOXYGEN_PARSING
#define _R(x) static_cast<oe::real_t>(x)
#endif

#include <float.h>

static const real_t RealEpsilon = FLT_EPSILON;					//!< Real number machine epsilon.
static const real_t RealPi = _R(3.14159265358979323433832792);	//!< Real number approximation to \f$ \pi \f$ .

//! Computes the square root of a positive real number.
extern inline real_t RealSqrt(const real_t& r) throw();

//! Computes the sine of a real number.
extern inline real_t RealSin(const real_t& r) throw();

//! Computes the inverse sine of a real number.
extern inline real_t RealInvSin(const real_t& r) throw();

//! Computes the cosine of a real number.
extern inline real_t RealCos(const real_t& r) throw();

//! Computes real number powers.
extern inline real_t RealPow(const real_t& x, const real_t& y) throw();

class Vector3;
class Quaternion;
class Matrix4x4;

} // namespace oe
