/*  OpenEngine: A free open source rendering system for DirectX.
	Copyright (C) 2002 Simon Brown

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#pragma once

/*! \mainpage

	\section intro Introduction
	\subsection whereami Where Am I?

	[Put some text here]

	\todo Actually write an introduction page.
*/

// windows
#define WIN32_LEAN_AND_MEAN
#define _WIN32_WINDOWS 0x0410
#include <windows.h>

// directx
#define DIRECTINPUT_VERSION 0x0800
#include <dinput.h>
#include <d3d8.h>
#include <dxerr8.h>
#include <d3dx8.h>

// stl
#include <iostream>
#include <sstream>
#include <fstream>
#include <iomanip>
#include <hash_map>
#include <vector>
#include <stdexcept>
#include <functional>
#include <algorithm>

// boost
#include <boost/scoped_ptr.hpp>
#include <boost/scoped_array.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/shared_array.hpp>
#include <boost/weak_ptr.hpp>
#include <boost/utility.hpp>

// usage of boost::non_copyable requires this in MSVC7
#pragma warning(disable: 4511 4512) // Copy constructor/assignment operator could not be generated. Good! :)

// openengine
#include "internals.h"
#include "shaderconstants.h"
#include "types.h"
#include "Exception.h"
#include "maths.h"
#include "Vector3.h"
#include "Quaternion.h"
#include "Matrix4x4.h"
#include "Projection.h"
#include "ComPtr.h"
#include "Texture.h"
#include "SharedHandle.h"
#include "SafeLock.h"
#include "VertexBuffer.h"
#include "IndexBuffer.h"
#include "FrameTimer.h"
#include "RenderWindow.h"
#include "GraphicsLoader.h"
#include "VertexShader.h"
#include "PixelShader.h"
#include "gui.h"
#include "Font.h"
#include "TextBuffer.h"
#include "Graphics.h"
#include "Material.h"
#include "Renderable.h"
#include "Scene.h"
#include "util.h"
#include "Input.h"